@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -16,12 +18,12 @@
  *  MA 02110-1301, USA.
  */
 
-#ifdef CLAMUKO
+#if defined(CLAMUKO) || defined(CLAMAUTH)
 
 #ifndef __CLAMUKO_H
 #define __CLAMUKO_H
 
-short int clamuko_scanning;
+extern short int clamuko_scanning;
 void *clamukoth(void *arg);
 
 #endif
