@@ -35,6 +35,7 @@
 #include "others.h"
 #include "mpool.h"
 #include "server.h"
+#include "libclamav/others.h"
 
 #ifdef HAVE_MALLINFO
 #include <malloc.h>
@@ -186,7 +187,7 @@ static void print_queue(int f, work_queue_t *queue, struct timeval *tv_now)
 		 (unsigned)queue->item_count);
 }
 
-int thrmgr_printstats(int f)
+int thrmgr_printstats(int f, char term)
 {
 	struct threadpool_list *l;
 	unsigned cnt, pool_cnt = 0;
@@ -301,7 +302,7 @@ int thrmgr_printstats(int f)
 		mdprintf(f,"MEMSTATS: heap N/A mmap N/A used N/A free N/A releasable N/A pools %u pools_used %.3fM pools_total %.3fM\n",
 			 pool_cnt, pool_used/(1024*1024.0), pool_total/(1024*1024.0));
 	}
-	mdprintf(f,"END\n");
+	mdprintf(f,"END%c", term);
 	pthread_mutex_unlock(&pools_lock);
 	return 0;
 }
@@ -467,7 +468,15 @@ threadpool_t *thrmgr_new(int max_threads, int idle_timeout, int max_queue, void
 #if defined(C_BIGSTACK)
 	pthread_attr_getstacksize(&(threadpool->pool_attr), &stacksize);
 	stacksize = stacksize + 64 * 1024;
-	if (stacksize < 1048576) stacksize = 1048576; /* at least 1MB please */
+	if (stacksize < 1048576) /* at least 1MB please */
+#if defined(C_HPUX) && defined(USE_MPOOL)
+		/* Set aside one cli_pagesize() for the stack's pthread header,
+		 * giving a 1M region to fit a 1M large-page */
+		if(cli_getpagesize() < 1048576)
+			stacksize = 1048576 - cli_getpagesize();
+		else
+#endif
+		stacksize = 1048576;
 	logg("Set stacksize to %lu\n", (unsigned long int) stacksize);
 	pthread_attr_setstacksize(&(threadpool->pool_attr), stacksize);
 #endif
