@@ -38,6 +38,8 @@
 #define LABDIFF(x) labdiff2(x)
 #endif
 
+/* #define LOGPARSEICONDETAILS */
+
 struct GICONS {
     unsigned int cnt;
     uint32_t lastg;
@@ -1498,6 +1500,7 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
 	} else
 	    confidence = (color + (gray + bright + noedge)*2/3 + dark + edge + colors) / 6;
 
+#ifdef LOGPARSEICONDETAILS
 	cli_dbgmsg("parseicon: edge confidence: %u%%\n", edge);
 	cli_dbgmsg("parseicon: noedge confidence: %u%%\n", noedge);
 	if(!bwmatch) {
@@ -1508,6 +1511,7 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
 	cli_dbgmsg("parseicon: dark confidence: %u%%\n", dark);
 	if(!bwmatch)
 	    cli_dbgmsg("parseicon: spread confidence: red %u%%, green %u%%, blue %u%% - colors %u%%\n", reds, greens, blues, ccount);
+#endif
 
 	if(confidence >= positivematch) {
 	    cli_dbgmsg("confidence: %u\n", confidence);
