@@ -23,10 +23,6 @@
 #include "clamav-config.h"
 #endif
 
-#ifndef CL_DEBUG
-#define NDEBUG
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,7 +45,7 @@ enum node_type {
 };
 
 struct node {
-	enum node_type type;
+	enum node_type type; /* must be first field */
 	struct node *parent;
 	union {
 		struct {
